#!/usr/bin/env python

import subprocess
from setuptools import setup, find_packages

try:
    version_git = subprocess.check_output(['git', 'describe', '--abbrev=0']).strip()
except:
    with open('version.py', 'r') as fh:
        version_git = open('version.py').read().split('=')[-1].strip()

with open('version.py', 'w') as fh:
    fh.write("# This file is managed by git\n" + "__version__ = " + version_git)

with open('requirements.txt') as fh:
    required = fh.read().splitlines()

setup(
    name='aws-tools',
    version=version_git,
    description='Switching between multiple AWS accounts & renewing API access keys',
    long_description=open('README.rst').read(),
    license='MIT',
    url='https://github.com/bartekj/aws-tools',
    keywords='aws key keys',
    packages=find_packages(),
    scripts=[
        'bin/aws-env-update.py',
        'bin/aws-roll-keys.py',
        'bin/aws_tools_completion.bash',
    ],
    install_requires=required,
    classifiers=(
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
    ),
)



#!/usr/bin/env python

import argparse
import os
import re
import sys
import gnupg
import logging
import getpass
import boto3
import smtplib
import datetime
from datetime import timedelta
from email.mime.application import MIMEApplication
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText


KEY_ID = "aws_access_key_id"
ACCESS_KEY = "aws_secret_access_key"

CREDENTIALS_FILE_TPL = """
[default]
aws_access_key_id = {id}
aws_secret_access_key = {key}
"""

home = os.environ["HOME"]
aws_config_dir = "{}/.aws/".format(home)

today = datetime.date.today()
future = today + datetime.timedelta(days=7)

def get_current_key(env, file_path, gpg, phrase):
    private_keys = gpg.list_keys(True)
    id, key = [None, None]
    if not private_keys:
        logging.error("No private key(s) found! Please check your GPG config")
        return [None, None]
    try:
        with open(file_path) as file:
            decrypted = gpg.decrypt_file(file, passphrase=phrase)
            if decrypted.status != "decryption ok":
                logging.error("Unable to decrypt {}".format(file_path))
                return [None, None]

            id = re.findall(r"{} = (.*)".format(KEY_ID), str(decrypted))[0]
            key = re.findall(r"{} = (.*)".format(ACCESS_KEY), str(decrypted))[0]
    except IOError:
        logging.warning("File for env {} not found. Skipping.".format(env))

    return [id, key]

def get_passphrase(use_agent=False):
    if use_agent:
        return None
    else:
        return getpass.getpass("Please enter passphrase for decrypting env files: ")

def get_smtp_conf(smtpconf, gpg, phrase):
    private_keys = gpg.list_keys(True)
    LOGIN, PASS, HOST, PORT, = [None, None, None, None]
    if not private_keys:
        logging.error("No private key(s) found! Please check your GPG config")
        return [None, None, None, None]
    try:
        with open(smtpconf) as file:
            decrypted = gpg.decrypt_file(file, passphrase=phrase)
            if decrypted.status != "decryption ok":
                logging.error("Unable to decrypt {}".format(smtpconf))
                return [None, None, None, None, None, None]

            LOGIN = re.findall(r"{} = (.*)".format("smtplogin"), str(decrypted))[0]
            PASS = re.findall(r"{} = (.*)".format("smtppass"), str(decrypted))[0]
            HOST = re.findall(r"{} = (.*)".format("smtphost"), str(decrypted))[0]
            PORT = re.findall(r"{} = (.*)".format("smtpport"), str(decrypted))[0]
    except IOError:
        logging.error("Can't open {0}".format(smtpconf))
        sys.exit(1)

    return [LOGIN, PASS, HOST, PORT]

def send(srv, sendto, data):
    try:
        server = smtplib.SMTP(srv[2], srv[3])
        server.set_debuglevel(False)
        server.ehlo()
        if server.has_extn('STARTTLS'):
            server.starttls()
            server.ehlo()
        server.login(srv[0], srv[1])
        server.sendmail(srv[0], sendto, data)
        server.quit()
        print "Sent to {0}".format(sendto)
    except Exception as exc:
        logging.error("Can't send email: {0}".format(exc))
        sys.exit(1)

def main():
    available_envs = list(
        map(lambda file: re.sub(r"env.(.*).conf.asc", r"\1", file),
            filter(lambda file: file.startswith("env"),
                   os.listdir(aws_config_dir))))
    parser = argparse.ArgumentParser(
        description="Rolls AWS IAM Access Keys for all or specified env(s)",
        epilog="Copyright (C) 2016 Karolis Labrencis <karolis@labrencis.lt>")
    parser.add_argument("-e", "--env", help="environment name", required=True,
                        choices=available_envs + ["all"])
    parser.add_argument("-s", "--send", help="Send an email with new keys to",
                        action="store", dest="sendkeysto")
    parser.add_argument("-i", "--info", help="Send info about rotation to",
                        action="store", dest="sendinfoto")
    parser.add_argument("-a", "--use-agent", action="store_true", help="Use GPG agent")
    parser.add_argument("--gpg-binary", help="GPG binary to use")
    parser.add_argument("-d", "--debug", action="store_true", help="Debug mode")
    parser.add_argument("-v", "--version", help="Print version", action="version",
                        version="%(prog)s 1.0")
    args = parser.parse_args()

    if args.debug:
        logging.basicConfig(format=logging.BASIC_FORMAT, level=logging.DEBUG)
        logging.info("Debug output.")

    if args.env == "all":
        args.env = available_envs
    else:
        args.env = [args.env]

    if args.gpg_binary is not None:
        gpg = gnupg.GPG(use_agent=args.use_agent, gpgbinary=args.gpg_binary)
    else:
        gpg = gnupg.GPG(use_agent=args.use_agent)
    phrase = get_passphrase(args.use_agent)

    msgkeys = MIMEMultipart()
    envs = ""
    msgbody = ""

    for env in args.env:
        file_path = os.path.join(os.sep, aws_config_dir, "env.{0}.conf.asc".format(env))
        key_id, access_key = get_current_key(env, file_path, gpg, phrase)

        if key_id is None:
            logging.warning("Skipping environment '{}'".format(env))
            continue

        client = boto3.client("iam", aws_access_key_id=key_id, aws_secret_access_key=access_key)
        current_key_id = \
            client.list_access_keys()["AccessKeyMetadata"][0]["AccessKeyId"]
        resp = client.create_access_key()
        new_id = resp["AccessKey"]["AccessKeyId"]
        new_key = resp["AccessKey"]["SecretAccessKey"]

        contents = CREDENTIALS_FILE_TPL.format(id=new_id, key=new_key)

        private_key = gpg.list_keys(True)
        encrypted = gpg.encrypt(contents, private_key[0]["uids"][0])

        with open(file_path, "w") as out:
            out.write(str(encrypted))

        client.delete_access_key(AccessKeyId=current_key_id)

        msgbody += "Rolled key for env {}: AccessKeyId={}; CreateDate={}\n".format(
            env, "*" * 16 + resp["AccessKey"]["AccessKeyId"][-5:],
            resp["AccessKey"]["CreateDate"]
        )

        if args.sendkeysto:
            with open(file_path, "rb") as attach:
                part1 = MIMEApplication(attach.read(), Name="env.{0}.conf.asc".format(env))
                part1["Content-Disposition"] = 'attachment; filename={}'.format(
                    "env.{0}.conf.asc".format(env))
                msgkeys.attach(part1)
                envs += " {0}".format(env)

    print msgbody
    vars = dict()

    if args.sendkeysto or args.sendinfoto:
        smtpconf = "{}/smtp.cfg.asc".format(aws_config_dir)
        srv = get_smtp_conf(smtpconf, gpg, phrase)

    if args.sendkeysto:
        msgkeys["To"] = args.sendkeysto
        msgkeys["From"] = srv[0]
        msgkeys["Subject"] = "AWS keys: {}".format(envs)

        send(srv, args.sendkeysto, msgkeys.as_string())

    if args.sendinfoto:
        msginfo = MIMEText(msgbody, "plain", "utf-8")
        msginfo["To"] = args.sendinfoto
        msginfo["From"] = srv[0]
        msginfo["Subject"] = "AWS weekly key(s) rotation: {0}-{1}".format(
            today.strftime("%Y%m%d"), future.strftime("%Y%m%d"))

        send(srv, args.sendinfoto, msginfo.as_string())


if __name__ == "__main__":
    main()

# vim: tabstop=4 expandtab shiftwidth=4 softtabstop=4 smarttab



#!/usr/bin/env python
import argparse
import getpass
import logging as log
import os
import re
import sys
import gnupg

debug = 0
home = os.environ['HOME']
aws_config_dir = '{}/.aws/'.format(home)
credential_file = '{0}/credentials'.format(aws_config_dir)
env_file = '{0}/.env'.format(aws_config_dir)

def get_args():
    '''This function parses and return arguments passed in'''
    available_envs = list(
        map(lambda file: re.sub(r"env.(.*).conf.asc", r"\1", file),
            filter(lambda file: file.startswith("env"),
                   os.listdir(aws_config_dir))))

    parser = argparse.ArgumentParser(__file__, formatter_class=argparse.RawDescriptionHelpFormatter,
                                    description=('''\
Simple script that will pick up gpg encrypted files from ~/.aws
and save them in the ${HOME}/.aws/credentials file\
Example usage:\n\n\tx1:~$ awsenv test \n'''),
                                    epilog="Copyright (C) 2016 Bart Jakubowski <bartekj@gmail.com>")
    parser.add_argument("-e", "--env", help="environment name", choices=available_envs, required=True)
    parser.add_argument("-a", "--use-agent", action="store_true", help="Use GPG agent")
    parser.add_argument("-x", "--export", action="store_true", help="Print eval-friendly output")
    parser.add_argument("--gpg-binary", help="GPG binary to use")
    parser.add_argument("-d", "--debug", action='store_true', help="Debug mode")
    parser.add_argument('-v', "--version", help="Print version", action='version', version='%(prog)s 1.0')
    args = parser.parse_args()
    return args.env, args.use_agent, args.export, args.gpg_binary, args.debug

def get_passphrase(use_agent=False):
    if use_agent:
        return None
    else:
        return getpass.getpass("Enter the passphrase to decrypt the env file: ")

def main():
    env, use_agent, export, gpg_binary, debug = get_args()

    if debug:
        log.basicConfig(format="%(levelname)s: %(message)s", level=log.DEBUG)
        log.info("Debug output.")
    else:
        log.basicConfig(format="%(levelname)s: %(message)s")

    encrypted_credentials_file = os.path.join(os.sep, aws_config_dir, 'env.{0}.conf.asc'.format(env))

    log.info('''Variables dump:\n\tenv : {0}\n\thome : {1}\n\taws_config_dir : {2}\n\tencrypted_credentials_file : {3}
    \tcredential_file : {4}'''.format(env, home, aws_config_dir, encrypted_credentials_file, credential_file))

    if not encrypted_credentials_file:
        log.error('''File with encrypted credentials for environment: {0} dont exists!.\n
                 Please encrypt your aws keys to file: \n{1}env.{0}.conf.asc'''.format(env, aws_config_dir))

    if gpg_binary is not None:
        gpg = gnupg.GPG(use_agent=use_agent, gpgbinary=gpg_binary)
    else:
        gpg = gnupg.GPG(use_agent=use_agent)

    private_keys = gpg.list_keys(True)

    if not private_keys:
        log.error('No private key(s) found! Please check your GPG config')

    stream = open(encrypted_credentials_file, "rb")
    output = gpg.decrypt_file(stream, passphrase=get_passphrase(use_agent))

    if use_agent and output.status != 'decryption ok':
        log.error('Decryption failed, please try again')
        sys.exit(1)
    else:
        while output.status != 'decryption ok':
            log.error('Decryption failed, check your password')
            stream.seek(0)
            output = gpg.decrypt_file(stream, passphrase=get_passphrase(use_agent))

    aws_credentials_patterns = ("aws_access_key_id", "aws_secret_access_key")

    if any(x in str(output) for x in aws_credentials_patterns):
        if not export:
            with open(credential_file, 'w') as credential_out:
                credential_out.write(str(output))
            with open(env_file, 'w') as env_out:
                env_out.write(env)

    else:
        log.error('No AWS credentials in the decrypted file!')

    stream.close()

    if export and output.status == "decryption ok":
        id = re.findall(r"{} = (.*)".format(aws_credentials_patterns[0]), str(output))[0]
        key = re.findall(r"{} = (.*)".format(aws_credentials_patterns[1]), str(output))[0]

        print("export AWS_ENV='{}'".format(env))
        print("export AWS_ACCESS_KEY_ID='{}'".format(id))
        print("export AWS_SECRET_ACCESS_KEY='{}'".format(key))

if __name__ == "__main__":
    main()

# vim: tabstop=4 expandtab shiftwidth=4 softtabstop=4 smarttab
